import style;
import graphical;
import draw_summary_mixing;

FloatingObject phantom_box() { return make_invisible(process_box("Input")); }

FloatingObject attention_box(bool with_text=true) {
    pair attention_box_size = process_size_for_lines(
        width_factor=1.75, num_lines=2.4);
    
    string label_text = "Self-attention or SummaryMixing";
    if (!with_text)
        label_text = "";

    FloatingObject attention_box = rounded_rectangle(
        attention_box_size, process_radius,
        p=attention_plate_base_colour,
        fillpen=make_phantom_pen(attention_plate_base_colour) + .5white,
        label_text=label_text, label_pen=defaultpen);

    return attention_box;
}

// Draw convolutional block next to the bottom of the attention block.
path draw_conv(path attention) {
    FloatingObject conv_box = process_box("cgMLP", width_factor=0.9);
    return draw_at((0,0),
        align(attention, S,
        next_to(attention,
        parallel_gap, conv_box, reference_bounding_box=true),
        reference_bounding_box=true));
}

struct InputVector {
    path input_vector;
    pair first_corner;
};

InputVector draw_input_vector(path[] receivers, FloatingObject vector,
    real distance = 2.5 * default_arrow_size)
{
    path input_vector = draw_next_to(
        combine(receivers), (0, -distance),
        vector, reference_bounding_box=true);
    
    path last_arrow;
    for (path receiver : receivers) {
        last_arrow = draw_manhattan_arrow(input_vector, receiver,
            start_direction=N, corner_from_start=1.*default_arrow_size);
    }

    InputVector result;
    result.input_vector = input_vector;
    result.first_corner = point(last_arrow, 1);
    return result;
}

/// Draw phantom input at the bottom with arrows to receivers.
InputVector draw_phantom_input(
    path[] receivers, real distance = 2.5 * default_arrow_size)
{
    return draw_input_vector(receivers, make_invisible(vector()));
}

path draw_output_vector(path source, FloatingObject vector) {
    path output_vector = draw_next_to(
        source, (0, + 2.5 * default_arrow_size),
        vector, reference_bounding_box=true);
    
    draw_arrow(source, output_vector);
    return output_vector;
}


/// Draw phantom input at the bottom with an arrow from a source.
path draw_phantom_output(path source) {
    return draw_output_vector(source, make_invisible(vector()));
}

// Draw residual plus and arrow around everything.
path draw_residual(pair arrow_start, path left_most, path last_output,
    bool with_phantom_output=true)
{
    path add_residual = draw_next_to(last_output, after_gap, plus_box());

    draw_arrow(last_output, add_residual);

    real residual_arrow_left = min(left_most).x - 0.2*reflen;
    path residual_arrow = arrow_start
        -- (residual_arrow_left, arrow_start.y)
        -- (residual_arrow_left, centre(add_residual).y)
        -- centre(add_residual);
    actually_draw_arrow(lastcut(residual_arrow, add_residual).before);

    if (with_phantom_output)
        draw_phantom_output(add_residual);

    return add_residual;
}

path draw_time_plates(picture[] behind, path content,
    pen base_pen,
    real left_margin=-1, real right_margin=-1, real bottom_margin=-1,
    bool numbering_left=false)
{
    path time_plate = draw_plate_around(behind[0],
        content,
        right_margin=right_margin,
        left_margin=left_margin,
        bottom_margin=bottom_margin,
        fillpen=make_phantom_pen(base_pen) + .5white,
        p=base_pen);

    path last_plate;
    for (int shadow=behind.length-1; shadow > 0; -- shadow) {
        path shadow_plate = shift(shadow * time_plate_offset) * time_plate;
        if (last_plate == nullpath)
            last_plate = shadow_plate;
        fill(behind[shadow], shadow_plate,
            .5*make_phantom_pen(base_pen) + .9white);
        draw(behind[shadow], shadow_plate,
            make_phantom_pen(base_pen));
    }
    if (numbering_left) {
        pair top_plate_corner = (min(time_plate).x, max(time_plate).y);
        pair bottom_plate_corner = (min(last_plate).x, max(last_plate).y);
        label(top_plate_corner, "$1$", align=SE);
        label(bottom_plate_corner, "$\Length$", align=NE);
    } else {
        pair top_plate_corner = (max(time_plate).x, max(time_plate).y);
        pair bottom_plate_corner = (max(last_plate).x, max(last_plate).y);
        label(top_plate_corner, "$1$", align=SW);
        label(bottom_plate_corner, "$\Length$", align=E);
        // label(.5*(top_plate_corner + bottom_plate_corner),
        //     "\reflectbox{$\ddots$}");
    }

    for (int shadow = 0; shadow < behind.length; ++ shadow) {
        add(currentpicture, behind[shadow], above=false);
    }
    return time_plate;
}

path draw_plates(picture[] behind, path extent) {
    return draw_time_plates(behind, extent, block_plate_base_colour,
        right_margin=0.1 * reflen,
        // Around arrows.
        left_margin=0.3 * reflen,
        bottom_margin=0.4 * reflen);
}
