import style;
import graphical;
import local_style;

real right_plate_margin_for_label = .2 * reflen;

/**
Return two closed halves of a cyclic path.
This may depend heavily on which direction the paths go in, so maybe it somehow
should be made general.
*/
path[] cut_cycle_in_two(path p, path knife) {
    real[][] points = intersections(p, knife);

    path first_segment = subpath(p, 0, points[0][0]);
    path middle_segment = subpath(p, points[0][0], points[1][0]);
    path last_segment = subpath(p, points[1][0], length(p));

    path knife_segment = subpath(knife, points[0][1], points[1][1]);

    path[] result = {
        middle_segment--reverse(knife_segment)--cycle,
        first_segment--knife_segment--last_segment--cycle
    };
    
    return result;
}

path[] cut_cycle_in_many(path p, path[] knives) {
    path[] result;
    path remaining = p;
    for (path knife: knives) {
        path[] two = cut_cycle_in_two(remaining, knife);
        result.push(two[0]);
        remaining = two[1];
    }
    result.push(remaining);
    return result;
}

FloatingObject[] vector_slices(int number, pair original_size=vector_size,
    pen p=defaultpen, pen fillpen=white)
{
    path original_blob = rounded_rectangle_shape(
        (0,0), original_size, vector_corner_radius);
    
    real knife_extent = max(original_blob).y*2;
    real right = max(original_blob).x;
    real left = min(original_blob).x;

    // One fewer knife than segments.
    path[] knives;
    for (int i = 0; i < number-1; ++ i) {
        real position = right - (right - left) / number * (i+1);
        path knife = (position, -knife_extent) -- (position, +knife_extent);
        knives.push(knife);
    }

    path[] segments = cut_cycle_in_many(original_blob, knives);

    FloatingObject[] os;

    for (path segment: segments) {
        path draw_transformed(picture pic, transform t) {
            path shape = t * segment;
            fill(shape, fillpen);
            draw(shape, p);
            return shape;
        }
        os.push(FloatingObject(segment, draw_transformed));
    }
    return os;
}

struct SummaryMixingParts {
    path[] input_functions;
    path input_vector; // TODO remove
    path left_of_concatenation; // TODO remove
    path output_vector;
    path combined_lower;
};

SummaryMixingParts draw_summary_mixing(
    FloatingObject transformation_box = function_box("$\Transformation$"),
    picture[] behind)
{
    FloatingObject input_vector_box = vector("$\InputVector_{\Time}$");

    path input_vector = input_vector_box.outline;

    FloatingObject summary_mixing_transformation_box = function_box("$\Summary$");
    FloatingObject sum_box = function_box("$\frac1{\Length}\!\sum$", size_up=1.7);

    FloatingObject[] transformation_boxes = {
        transformation_box, next_to(
            transformation_box, -parallel_gap, summary_mixing_transformation_box)};
    // path[] transformations = draw_next_to(input_vector,
    //     1.2*after_gap, transformation_boxes, object_bounding_box=true);
    path[] transformations = draw_at((0,0), transformation_boxes);

    path single_summary_final = transformations[1];
    // label_outside(transformations[1], "Local\\summary", direction=N);

    transform time_arrow_spacing = shift(0, 0.15 * reflen);
    FloatingObject moved_sum_box = next_to(
        single_summary_final, -2 * parallel_gap, sum_box);
    path sum = draw_at((0,0),
        time_arrow_spacing * moved_sum_box);
    // label_outside(sum, "Global\\summary", E);
    {
        path a = draw_arrow(single_summary_final, moved_sum_box.outline,
            pen_for_source[0], "$1$",
            axis_aligned=true,
            label_position=.8, label_align=S);
        actually_draw_arrow(behind[0], time_arrow_spacing * a,
            pen_for_source[1]);
        actually_draw_arrow(behind[1],
            time_arrow_spacing * time_arrow_spacing * a,
            pen_for_source[2],
            "$\Length$", label_position=.8, label_align=N);
    }

    FloatingObject[] concatenation_slices = vector_slices(2);
    // Put the local block above transformations[0], for a vertical arrow.
    pair global_local_location_1 = location_next_to(
        transformations[0], 1*after_gap,
        concatenation_slices[1]);
    path[] global_locals = draw_at(
        global_local_location_1, concatenation_slices);

    path global_local = combine(global_locals);

    draw_arrow(transformations[0], global_locals[1]);

    {
        path feedback_line = cut_line_from_to(
            centre(sum) -- (centre(sum).x, centre(global_locals[0]).y)
            -- centre(global_locals[0]), sum, global_locals[0]);
        actually_draw_arrow(feedback_line);
        actually_draw_arrow(behind[0], time_arrow_spacing * feedback_line);
        actually_draw_arrow(behind[1],
            time_arrow_spacing * time_arrow_spacing * feedback_line);
    }

    path combiner = draw_next_to(global_local, after_gap,
        function_box("$\CombineFunction$"));

    draw_arrow(global_local, combiner);

    SummaryMixingParts result;
    result.input_functions = transformations;
    result.input_vector = input_vector;
    result.left_of_concatenation = global_locals[1];
    result.output_vector = combiner;
    result.combined_lower = combine(
        single_summary_final, combine(transformations));

    return result;
}
