import style;
import graphical;
import local_style;
import branchformer_helpers;

real source_step = 0.07 * reflen;
real target_step = 0.25 * reflen;


picture[] behind = {new picture, new picture, new picture};
picture[] layers = new picture[] {currentpicture,
    behind[0], behind[1], behind[2]};


FloatingObject[] input_transformations = {
    function_box("$q$"), function_box("$k$"), function_box("$v$")
};

path make_query = draw_at((0,0), input_transformations[0]);
InputVector phantom_input = draw_input_vector(new path[]{make_query},
    vector("$\InputVector_{\Time}$"), 0.8 * reflen);

path key_query_dot = draw_next_to(make_query, -parallel_gap,
    matmul_box());
path softmax = draw_next_to(key_query_dot, after_gap,
    process_box("Softmax"));

path value_dot = draw_next_to(softmax, after_gap,
    matmul_box());

draw_arrow(make_query, key_query_dot);
draw_arrow(key_query_dot, softmax);
draw_arrow(softmax, value_dot);
draw_output_vector(value_dot, vector("$\OutputVector_{\Time}$"));

pair first_corner = phantom_input.first_corner + (3.5 * reflen, 0);

path line_part_1 = phantom_input.first_corner--first_corner;


// From first time to others.
pair second_corner = (first_corner.x, centre(key_query_dot).y - source_step);

pair target = (centre(key_query_dot).x, second_corner.y);

pair value_first_corner = target + (1.3 * reflen, 0);
pair value_second_corner = (
    value_first_corner.x, centre(value_dot).y - source_step);
pair value_target = (centre(value_dot).x, value_second_corner.y);

for (int source_layer_index = 2; source_layer_index >= 0; -- source_layer_index)
{
    real source_offset = source_layer_index * source_step;
    transform source_shift = shift(source_offset, source_offset);
    pen layer_source_pen = pen_for_source[source_layer_index];

    // Start of each line.
    draw(layers[source_layer_index], source_shift * line_part_1,
        layer_source_pen);
    // Label the lines.
    if (source_layer_index == 0) {
        label(first_corner, "$1$", align=SW);
    }
    if (source_layer_index == 2) {
        label(shift(0, source_offset) * first_corner,
            "$\Length$", align=NW, layer_source_pen);
    }
    

    for (int target_layer_index = 0; target_layer_index < 3;
        ++ target_layer_index)
    {
        // End of each line to each time.
        real target_offset = target_layer_index * target_step;

        path line = source_shift * (
            first_corner--shift(0, target_offset) * (second_corner--target));
        actually_draw_arrow(layers[target_layer_index],
            lastcut(line, key_query_dot).before,
            layer_source_pen);
        
    }

    // Split line to make the value.
    path value_line = source_shift * (
        value_first_corner -- value_second_corner -- value_target);
    actually_draw_arrow(lastcut(value_line, value_dot).before,
        layer_source_pen);
}


path make_key = draw_next_to(key_query_dot, -parallel_gap,
    input_transformations[1]);
path make_key = draw_next_to(value_dot, -parallel_gap,
    input_transformations[2]);


draw_time_plates(behind,
    combine(combine(make_query, softmax), combine(value_dot, make_key)),
    attention_plate_base_colour,
    right_margin=0.65 * reflen,
    bottom_margin=0.7 * reflen,
    numbering_left=false);
