import graph;
import style;
import general;

void remote_label(System system, real[][][] datas,
    picture[] pics,
    transform[] frame_transforms, real[] connections_x,
    pair[] label_connection_offsets,
    pair label_location, pair label_align=(0,0))
{
    pen darker_pen = 0.7*system.colour;
    label(label_location, system.label, darker_pen, align=label_align);
    for (int i = 0; i < pics.length; ++ i) {
        transform pic_transform = pics[i].calculateTransform();

        pair graph_connection = frame_transforms[i] * pic_transform *
            Scale(pics[i],
                position_on_graph(datas[i][0], datas[i][1], connections_x[i]));


        dot(graph_connection, system.colour);
        draw(graph_connection--(label_location+label_connection_offsets[i])
            , darker_pen);
    }
}

/** Training **/

picture training;

size(training, 4.2cm, 4cm, IgnoreAspect);

real[][] branchformer_training_data = draw_system_no_label(
    training, "_time", branchformer);
real[][] contextnet_training_data = draw_system_no_label(
    training, "_time", contextnet);
real[][] fastformer_training_data = draw_system_no_label(
    training, "_time", fastformer);
real[][] summary_mixing_training_data = draw_system_no_label(
    training, "_time", summary_mixing);
real[][] summary_mixing_lite_training_data = draw_system_no_label(
    training, "_time", summary_mixing_lite);

xaxis(training, "Utterance length (s)", YZero(extend=false), LeftTicks);
yaxis(training, /*"Training time per sentence (s)",*/ ymin=0, ymax=0.8, Left, RightTicks);




/** Decoding **/

picture decoding;

size(decoding, 3.8cm, 4cm, IgnoreAspect);

real data_scale = 1e-4 * 1e5;

real[][] branchformer_decoding_data = draw_system_no_label(
    decoding, "_rtf", branchformer, data_scale=data_scale);
real[][] fastformer_decoding_data = draw_system_no_label(
    decoding, "_rtf", fastformer, data_scale=data_scale);
real[][] summary_mixing_decoding_data = draw_system_no_label(
    decoding, "_rtf", summary_mixing, data_scale=data_scale);
real[][] summary_mixing_lite_decoding_data = draw_system_no_label(
    decoding, "_rtf", summary_mixing_lite, data_scale=data_scale);

real y_min = 0.3 * data_scale;

xaxis(decoding, "Utterance length (s)", YEquals(y=y_min, extend=false), LeftTicks);
yaxis(decoding, /*"Real-time factor ($10^{-5}$)",*/ ymin=y_min, Right, RightTicks);


frame training_frame = training.fit();
frame decoding_frame = decoding.fit();

pair half_gap = (0.6cm, 0);
// The frames would normally be aligned at (0,0) in graph coordinates, which is
// useless here.
transform training_frame_transform = shift(
    -(max(training_frame).x, min(training_frame).y) - half_gap);
transform decoding_frame_transform = shift(half_gap - min(decoding_frame));

add(training_frame_transform * training_frame);
add(decoding_frame_transform * decoding_frame);

label_top_of_frame(training_frame, training_frame_transform,
    "Training time per sentence (s)");
label_top_of_frame(decoding_frame, decoding_frame_transform,
    "Real-time factor, decoding ($10^{-5}$)");

/** Labels **/

picture[] pictures = {training, decoding};
transform[] transforms = {training_frame_transform, decoding_frame_transform};
pair[] short_label_connection_offsets = {(-.5cm, 0), (+.5cm, 0)};
pair[] shorter_label_connection_offsets = {(-.6cm, 0), (+.6cm, 0)};
pair[] label_connection_offsets = {(-.8cm, 0), (+.8cm, 0)};
pair[] long_label_connection_offsets = {(-1.0cm, 0), (+1.0cm, 0)};

remote_label(branchformer,
    new real[][][]{branchformer_training_data, branchformer_decoding_data},
    pictures, transforms,
    new real[]{95, 40}, short_label_connection_offsets,
    label_location=(0cm, 3.7cm));

remote_label(contextnet,
    new real[][][]{contextnet_training_data},
    new picture[] {training}, transforms,
    new real[]{95, 15}, label_connection_offsets,
    label_location=(0cm, 2.85cm));

remote_label(summary_mixing,
    new real[][][]{summary_mixing_training_data, summary_mixing_decoding_data},
    pictures, transforms,
    new real[]{98, 15}, shorter_label_connection_offsets,
    label_location=(0cm, 2.3cm));

remote_label(fastformer,
    new real[][][]{fastformer_training_data, fastformer_decoding_data},
    pictures, transforms,
    new real[]{80, 28}, label_connection_offsets,
    label_location=(0cm, 1.75cm));

remote_label(summary_mixing_lite,
    new real[][][]{summary_mixing_lite_training_data, summary_mixing_lite_decoding_data},
    pictures, transforms,
    new real[]{75, 20}, long_label_connection_offsets,
    label_location=(0cm, 1.2cm));

