import graph;
import style;
import general;

void remote_label(picture pic, System system, real[][] data,
    real connection_x, pair label_location, pair label_connection_offset,
    pair label_align=(0,0))
{
    pen darker_pen = 0.7*system.colour;
    transform pic_transform = pic.calculateTransform();

    pair graph_connection = pic_transform * Scale(pic, position_on_graph(
        data[0], data[1], connection_x));

    dot(graph_connection, system.colour);
    label(label_location, system.label, darker_pen, align=label_align);
    draw(graph_connection--(label_location+label_connection_offset), darker_pen);
}


picture vram_graph;

size(vram_graph, 4cm, 4cm, IgnoreAspect);

real[][] branchformer_data = draw_system_no_label(
    vram_graph, "_vram", branchformer);
real[][] contextnet_data = draw_system_no_label(
    vram_graph, "_vram", contextnet);
real[][] fastformer_data = draw_system_no_label(
    vram_graph, "_vram", fastformer);
real[][] summary_mixing_data = draw_system_no_label(
    vram_graph, "_vram", summary_mixing);
real[][] summary_mixing_lite_data = draw_system_no_label(
    vram_graph, "_vram", summary_mixing_lite);

xaxis(vram_graph, "Utterance length (s)", YZero(extend=false), LeftTicks);
yaxis(vram_graph, /*"Peak VRAM, training (GB)",*/ ymin=0, ymax=60, Right, RightTicks);


frame vram_frame = vram_graph.fit();
add(vram_frame);

label_top_of_frame(vram_frame, shift(0),
    "Peak VRAM, training (GB)");

remote_label(vram_graph, branchformer, branchformer_data,
    90, label_location=(2cm, 3cm), label_connection_offset=(0.3cm, -0.2cm));

remote_label(vram_graph, summary_mixing, summary_mixing_data,
    90, label_location=(1.7cm, 2.4cm), label_connection_offset=(0cm, 0),
    label_align=W);
remote_label(vram_graph, fastformer, fastformer_data,
    80, label_location=(1.7cm, 2.0cm), label_connection_offset=(0cm, 0),
    label_align=W);
remote_label(vram_graph, contextnet, contextnet_data,
    70, label_location=(1.7cm, 1.6cm), label_connection_offset=(0cm, 0),
    label_align=W);
remote_label(vram_graph, summary_mixing_lite, summary_mixing_lite_data,
    52, label_location=(1.7cm, 1.2cm), label_connection_offset=(-0.3cm, -0.2cm),
    label_align=W);
