/**
Constants for style.
*/

real reflen = 0.9cm;

usepackage("mathtools");
usepackage("mathdots");
texpreamble("\input{symbols}");

defaultpen(defaultpen + linewidth(0.2mm) + fontsize(9) + TimesRoman());
pen graph_line_pen = defaultpen + linewidth(0.3mm);
real default_arrow_size = .2reflen;
pen arrow_pen = defaultpen /*+ linewidth(.7*linewidth(defaultpen))*/;

// Finite-state diagrams

pen finite_state_pen = defaultpen + linewidth(0.2mm);
pen finite_state_start_pen = finite_state_pen + linewidth(0.5mm);
real finite_state_arrow_size = default_arrow_size;

real finite_state_radius = 4.5mm;
real finite_state_inner_radius = finite_state_radius - 0.25mm;
real finite_state_outer_radius = finite_state_radius + 0.25mm;

pair finite_state_horizontal = (2.5reflen, 0);
pair finite_state_vertical = (0, 2reflen);

// Flow diagrams.

real process_radius = .15reflen;
real vector_corner_radius = .03reflen;

real plate_margin = 0.07reflen;

pair process_step = (0, 1.1reflen);
pair process_gap = (0, 2 * default_arrow_size);

pair process_size_for_lines(real num_lines=1, real width_factor=1.0) {
    return (width_factor * 1.5reflen, .1reflen + num_lines * .4reflen);
}

pair process_size = process_size_for_lines(1);

// Size of text without border.
pair unboxed_size = (1.5reflen, .4reflen);

// Neural diagrams.

pair vector_size = (.4reflen, 0.85reflen);
pair time_step = (.55reflen, 0);
pair time_gap = (.1reflen, 0);

real atomic_operation_radius = 0.25reflen;

// Samsung colours.

pen samsung_blue = RGB(20, 40, 160);

pen[] other_colours = {
    RGB(0, 119, 200),  // Sky Blue
    RGB(0, 179, 227),  // Sea Blue
    RGB(0, 195, 178),  // Teal
    RGB(28, 147, 220), // Lavender
    RGB(151, 214, 83), // Clover
    RGB(255, 181, 70), // Saffron
    RGB(255, 67, 55)   // Coral
};

// Other colours.

pen make_phantom_pen(pen base_pen) {
    return 0.3*base_pen + .7*white /* + linetype(new real[]{0, 2})*/;
}

pen make_shade_box_pen(pen base_pen, real shadiness=1.0) {
    return base_pen + opacity(0.2 / shadiness);
}

pen phantom_pen = make_phantom_pen(defaultpen);
pen highlight_pen = defaultpen + other_colours[5];
pen shade_box_pen = make_shade_box_pen(samsung_blue);
pen shade_box_label_pen = samsung_blue;
